use chrono::Utc;
use domain::{
    dto::account::AuthenticateWithPassword,
    entities::account::{Account, Role}, vo::account::LoginAccount,
};
use library::{
    cache::account_cache::{CacheAccount, LOGIN_CACHE}, db, resp::response::{ResErr, ResData, ResResult}, token::{generate_refresh_token, generate_token}
};


pub async fn authenticate_with_password(
    req: AuthenticateWithPassword,
) -> ResResult<ResData<LoginAccount>> {

    let account =
        Account::find_with_password(req.username.unwrap(), req.password.unwrap(), db!()).await?;
    if account.is_none() {
        tracing::info!("登录用户失败，用户查询为空");
        return Err(ResErr::params("用户名或密码错误"));
    }
    let account = account.unwrap();
    if account.disable_time > Utc::now() {
        tracing::error!("账户已禁用");
        return Err(ResErr::auth("账户已禁用"));
    }
    if account.role != Role::Admin {
        tracing::error!("账户不是管理员，无权限");
        return Err(ResErr::perm("账户无权限"));
    }

    let token = generate_token(&account.id);
    let refresh_token = generate_refresh_token(&account.id);

    LOGIN_CACHE
        .insert(
            account.id.to_owned(),
            CacheAccount {
                account: account.clone(),
                token: token.to_owned(),
            },
        )
        .await;

    let login_account = LoginAccount{ 
        username: account.username, 
        display_name: account.display_name, 
        avatar_url: account.avatar_url, 
        metadata: account.metadata, 
        wallet: account.wallet, 
        email: account.email, 
        token, 
        refresh_token  
    };
    Ok(ResData::some(login_account))
}

pub async fn add_account() -> ResResult<ResData<()>> {
    let account = Account::add_account(
        &Account {
            username: "admin".to_string(),
            password: Some("123456".as_bytes().to_vec()),
            role: Role::Admin,
            ..Default::default()
        },
        db!(),
    )
    .await?;
    if account.is_none() {
        tracing::error!("添加用户失败");
        return Err(ResErr::service("添加用户失败"));
    }
    Ok(ResData(None))
}